"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _RQElementContainer = _interopRequireDefault(require("./RQElementContainer"));

var _AppRegistry = _interopRequireDefault(require("../common/AppRegistry"));

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var contextRoots = new Map();

var createHostContext = function createHostContext(rootContainerInstance) {
  var qml = "import QtQuick 2.7; Item { visible: false; }";
  var fakeRoot = Qt.createQmlObject(qml, rootContainerInstance.element, 'HostContext');
  return new _RQElementContainer.default(fakeRoot, {
    defaultProp: 'data'
  });
};

function getHostContext(rootContainerInstance) {
  var rootContext = contextRoots.get(rootContainerInstance);

  if (!rootContext) {
    rootContext = createHostContext(rootContainerInstance);
  }

  return rootContext;
}

function createElementContainer(type, props, rootContainerInstance, hostContext) {
  var hostElement = hostContext.element;

  var rawComponentDefinition = _AppRegistry.default.getRawComponent(type);

  if (rawComponentDefinition) {
    var rawContent = rawComponentDefinition.rawContent,
        metadata = rawComponentDefinition.metadata;
    var element = Qt.createQmlObject(rawContent, hostElement, type);
    var container = new _RQElementContainer.default(element, metadata);
    container.setNativeProps(props);
    return container;
  }

  var componentDefinition = _AppRegistry.default.getComponent(type);

  if (componentDefinition) {
    var component = componentDefinition.component,
        _metadata = componentDefinition.metadata;

    var _element = component.createObject(hostElement, props);

    if (!_element) {
      throw new Error("Unable to create element: ".concat(type));
    }

    var _container = new _RQElementContainer.default(_element, _metadata);

    _container.setNativeProps(props);

    return _container;
  }

  throw new Error("Unknown type ".concat(type));
}

function appendChild(parentContainer, childContainer) {
  var parent = parentContainer.element;
  var child = childContainer.element;
  var parentType = (0, _utils.getObjectType)(parent);
  var childType = (0, _utils.getObjectType)(child); // special cases
  // - add MenuBar to ApplicationWindow

  if (childType === 'QQuickMenuBar' && parentType === 'QQuickApplicationWindow') {
    parent.menuBar = child;
    return;
  } // - sync QQuickPlatformMenu


  if (childType === 'QQuickPlatformMenu' && parentType === 'QQuickPlatformMenuBar') {
    parent.addMenu(child); // this is a hack to "sync" menu

    child.visible = false;
    child.visible = true;
    return;
  } // - QQuickPlatformMenuBar requires manually setting window


  if (childType === 'QQuickPlatformMenuBar' && (0, _utils.isWindow)(parent)) {
    child.window = parent;
  } // - set animation's target to parent, if not specified


  if ((0, _utils.isAnimation)(child) && !(0, _utils.isAnimation)(parent) && (0, _utils.isQuickItem)(parent)) {
    child.target = parent;
  }

  if ((0, _utils.isQuickItem)(parent) && (0, _utils.isQuickItem)(child)) {
    child.parent = parent;
    return;
  } // append child to parent's default prop


  var parentDefaultProp = parentContainer.metadata.defaultProp;
  var parentData = parent[parentDefaultProp];

  if (parentData && parentData.push) {
    parentData.push(child);
  }
}

function removeAllChildren(rootContext) {
  var defaultProp = rootContext.metadata.defaultProp;
  var data = rootContext.element[defaultProp];

  if (data && data.length > 0) {
    // destroy all children
    for (var index = data.length; index > 0; index--) {
      var child = data[index - 1];
      removeChildElement(rootContext, child);
    }
  }
}

function removeChild(parentContainer, childContainer) {
  return removeChildElement(parentContainer, childContainer.element);
}

function removeChildElement(parentContainer, child) {
  var parent = parentContainer.element;
  var parentDefaultProp = parentContainer.metadata.defaultProp;
  var parentType = (0, _utils.getObjectType)(parent);
  var childType = (0, _utils.getObjectType)(child);

  if ((0, _utils.isQuickItem)(parent) && (0, _utils.isQuickItem)(child)) {
    child.parent = null;
  } else {
    var parentData = parent[parentDefaultProp];

    if (parentData) {
      var childIndex = (0, _utils.findChildIndex)(parentData, child);

      if (childIndex > -1) {
        if (parent.remove) {
          parent.remove(childIndex);
        } else {}
      }
    }
  }

  child.destroy();
} // TODO: fix this


function insertBefore(parent, child, beforeChild) {
  var parentData = parent.element[parent.metadata.defaultProp];

  if (parentData) {
    // there is no indexOf for qml list, sad
    // @see https://stackoverflow.com/questions/43030433/qtquick-item-children-indexof-doesnt-exist
    var childIndex = (0, _utils.findChildIndex)(parentData, child.element);
    var beforeChildIndex = (0, _utils.findChildIndex)(parentData, beforeChild.element); // Move existing child or add new child?

    if (childIndex >= 0) {// move from childIndex to beforeChildIndex
      // moveChild(parentData, childIndex, beforeChildIndex);
    } else {
      // insert child into beforeChildIndex first
      appendChild(parent, child); // then move
      // moveChild(parentData, parentData.length - 1, beforeChildIndex);
    }
  }
}

var UIManager = {
  getHostContext: getHostContext,
  createElementContainer: createElementContainer,
  appendChild: appendChild,
  removeAllChildren: removeAllChildren,
  removeChild: removeChild,
  insertBefore: insertBefore
};
var _default = UIManager;
exports.default = _default;